<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\State;
use Magento\Store\Api\StoreConfigManagerInterface;

/**
 * Class Translations
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Translations extends AbstractExporter
{
    /**
     * @var \Magento\Framework\Translate
     */
    private $translate;

    /**
     * @var StoreConfigManagerInterface
     */
    private $storeConfigManager;

    /**
     * @var State
     */
    private $state;

    /**
     * Translations constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Translate\ResourceInterface $translate
     * @param StoreConfigManagerInterface $storeConfigManager
     * @param State $state
     * @param Mongo $mongo
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Translate\ResourceInterface $translate,
        StoreConfigManagerInterface $storeConfigManager,
        State $state,
        Mongo $mongo
    )
    {
        parent::__construct($mongo, $scopeConfig);
        $this->translate = $translate;
        $this->storeConfigManager = $storeConfigManager;
        $this->state = $state;

    }

    /**
     * @return iterable
     */
    public function getCollections(): iterable
    {
        yield 'translations';
    }

    /**
     * @throws \MongoDB\Driver\Exception\Exception
     */
    public function execute()
    {

        $id = 1;

        foreach ($this->getLocales() as $locale) {

            $bulkData = [];
            $data = $this->translate->getTranslationArray(null, $locale);

            //
            // TODO : discover which kind of label breaks json
            //
            // $data = $this->state->emulateAreaCode(Area::AREA_FRONTEND, function () use ($locale) {
            // return $this;
            //     return $this->translate->setLocale($locale)->loadData(Area::AREA_FRONTEND)->getData();
            // });

            foreach ($data as $string => $translation) {
                $bulkData[] = [
                    '_id'         => $id++,
                    'string'      => $string,
                    'translation' => strtr($translation, ['&#039;' => '\'']),
                    'locale_code' => $locale
                ];
            }

            $this->write($bulkData);
        }
    }

    /**
     * @return array
     */
    public function getLocales()
    {
        $locales = [];

        foreach ($this->storeConfigManager->getStoreConfigs() as $storeConfig) {
            $locales[] = $storeConfig->getLocale();
        }

        return array_unique($locales);
    }

    /**
     * @param array $bulkData
     */
    public function write(array $bulkData)
    {
        if (empty($bulkData)) {
            return;
        }

        try {
            $this->getConnection()->update('_id', $bulkData, 'translations', true, $this->getDatabase());
        } catch (\MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }

        return;
    }
}
